// Copyright (C) 2000, Luca Padovani <luca.padovani@cs.unibo.it>.
// 
// This file is part of GtkMathView, a Gtk widget for MathML.
// 
// GtkMathView is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// GtkMathView is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with GtkMathView; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 
// For details, see the GtkMathView World-Wide-Web page,
// http://cs.unibo.it/~lpadovan/mml-widget, or send a mail to
// <luca.padovani@cs.unibo.it>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <assert.h>
#include <stddef.h>

#include "FontAttributes.hh"

FontAttributes::FontAttributes()
{
  mask = 0;
  foundry = NULL;
  family = NULL;
  charSet = NULL;
}

// the following constructor is used to create a set of specialized attributes
// according to a given mask M. There is preference to take attributes from
// SOURCE, but if they are not available then they are taken as defaults from
// DEF (if specified there)
FontAttributes::FontAttributes(unsigned m, const FontAttributes& source, const FontAttributes& def)
{
  assert((m & source.mask) == m);

  mask = m | def.mask;

  if ((m & FONT_HAS_FOUNDRY) != 0) foundry = source.foundry;
  else if (def.HasFoundry()) foundry = def.foundry;

  if ((m & FONT_HAS_FAMILY) != 0) family = source.family;
  else if (def.HasFamily()) family = def.family;

  if ((m & FONT_HAS_STYLE) != 0) style = source.style;
  else if (def.HasStyle()) style = def.style;

  if ((m & FONT_HAS_WEIGHT) != 0) weight = source.weight;
  else if (def.HasWeight()) weight = def.weight;

  if ((m & FONT_HAS_SIZE) != 0) size = source.size;
  else if (def.HasSize()) size = def.size;

  if ((m & FONT_HAS_CHARSET) != 0) charSet = source.charSet;
  else if (def.HasCharSet()) charSet = def.charSet;
}

bool
FontAttributes::Equals(const FontAttributes& fa) const
{
  if (mask != fa.mask) return false;
  return CompatibleWith(fa);
}

bool
FontAttributes::CompatibleWith(const FontAttributes& fa) const
{
  return CompatibleWith(mask & fa.mask, fa);
}

bool
FontAttributes::CompatibleWith(unsigned m, const FontAttributes& fa) const
{
  assert((m & fa.mask) == m);

  unsigned n = m & mask;

  if ((n & FONT_HAS_FOUNDRY) != 0 && strcmp(foundry, fa.foundry)) return false;
  if ((n & FONT_HAS_FAMILY) != 0 && strcmp(family, fa.family)) return false;
  if ((n & FONT_HAS_STYLE) != 0 && style != fa.style) return false;
  if ((n & FONT_HAS_WEIGHT) != 0 && weight != fa.weight) return false;
  if ((n & FONT_HAS_SIZE) != 0 && !scaledEq(size.ToScaledPoints(), fa.size.ToScaledPoints())) return false;
  if ((n & FONT_HAS_CHARSET) != 0 && strcmp(charSet, fa.charSet)) return false;

  return true;
}
