// Copyright (C) 2000, Luca Padovani <luca.padovani@cs.unibo.it>.
// 
// This file is part of GtkMathView, a Gtk widget for MathML.
// 
// GtkMathView is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// GtkMathView is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with GtkMathView; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 
// For details, see the GtkMathView World-Wide-Web page,
// http://cs.unibo.it/~lpadovan/mml-widget, or send a mail to
// <luca.padovani@cs.unibo.it>

#ifndef FontAttributes_hh
#define FontAttributes_hh

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "keyword.hh"
#include "UnitValue.hh"

typedef unsigned FontAttributeMask;

#define FONT_HAS_FOUNDRY (1 << 0) // (5, never discarded)
#define FONT_HAS_FAMILY  (1 << 1) // (4, last discarded)
#define FONT_HAS_STYLE   (1 << 2) // (2)
#define FONT_HAS_WEIGHT  (1 << 3) // (1)
#define FONT_HAS_SIZE    (1 << 4) // (3)
#define FONT_HAS_CHARSET (1 << 5) // (0, first discarded)

struct FontAttributes {
  FontAttributes(void);
  FontAttributes(unsigned, const FontAttributes&, const FontAttributes&);

  bool HasFoundry(void) const { return (mask & FONT_HAS_FOUNDRY) != 0; }
  bool HasFamily(void) const { return (mask & FONT_HAS_FAMILY) != 0; }
  bool HasStyle(void) const { return (mask & FONT_HAS_STYLE) != 0; }
  bool HasWeight(void) const { return (mask & FONT_HAS_WEIGHT) != 0; }
  bool HasSize(void) const { return (mask & FONT_HAS_SIZE) != 0; }
  bool HasCharSet(void) const { return (mask & FONT_HAS_CHARSET) != 0; }

  bool Equals(const FontAttributes&) const;
  bool CompatibleWith(const FontAttributes&) const;
  bool CompatibleWith(unsigned, const FontAttributes&) const;

  unsigned     mask;

  const char*  foundry;
  const char*  family;
  UnitValue    size; // cannot be < 0
  FontWeightId weight;
  FontStyleId  style;
  const char*  charSet;
};

#endif
