// Copyright (C) 2000, Luca Padovani <luca.padovani@cs.unibo.it>.
// 
// This file is part of GtkMathView, a Gtk widget for MathML.
// 
// GtkMathView is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// GtkMathView is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with GtkMathView; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 
// For details, see the GtkMathView World-Wide-Web page,
// http://cs.unibo.it/~lpadovan/mml-widget, or send a mail to
// <luca.padovani@cs.unibo.it>

#ifndef GraphicsContext_hh
#define GraphicsContext_hh

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gdk/gdk.h>

#include "RGBValue.hh"
#include "Rectangle.hh"
#include "FontAttributes.hh"

enum LineStyle {
  LINE_STYLE_NOTVALID = -1,

  LINE_STYLE_SOLID,
  LINE_STYLE_DASHED
};

struct GraphicsContextValues {
  RGBValue  color;
  RGBValue  background;
  LineStyle lineStyle;
  Rectangle clipRectangle;
};

enum GraphicsContextValuesMask {
  GC_MASK_COLOR      = 1 << 0,
  GC_MASK_BACKGROUND = 1 << 1,
  GC_MASK_LINESTYLE  = 1 << 2,
  GC_MASK_CLIPRECT   = 1 << 3
};

class GraphicsContext {
public:
  GraphicsContext(GdkWindow*, GdkColormap*,
		  const GraphicsContextValues* = NULLPTR,
		  unsigned = 0);
  GraphicsContext(GdkGC*);
  ~GraphicsContext();

  GraphicsContext* Clone(void) const;

  void 	       SetLineStyle(LineStyle, unsigned = 1);
  void 	       SetColor(RGBValue);
  void 	       SetBackground(RGBValue);
  void         SetClipRectangle(scaled, scaled, scaled, scaled);
  void 	       SetClipRectangle(const Rectangle*);

  void         GetLineStyle(LineStyle*, unsigned*) const;
  unsigned     GetLineWidth(void) const;
  RGBValue     GetColor(void) const;
  RGBValue     GetBackground(void) const;
  const Rectangle& GetClipRectangle(void) const;

  GdkGC* GetNativeGraphicsContext(void) const { return gdk_gc; }

private:
  GraphicsContext(void);

  // device dependent fields
  GdkWindow*   gdk_window;
  GdkGC*       gdk_gc;
  GdkColormap* gdk_colormap;
};

#endif
