// Copyright (C) 2000, Luca Padovani <luca.padovani@cs.unibo.it>.
// 
// This file is part of GtkMathView, a Gtk widget for MathML.
// 
// GtkMathView is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// GtkMathView is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with GtkMathView; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 
// For details, see the GtkMathView World-Wide-Web page,
// http://cs.unibo.it/~lpadovan/mml-widget, or send a mail to
// <luca.padovani@cs.unibo.it>

#ifndef MathMLStretchyCharNode_hh
#define MathMLStretchyCharNode_hh

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "CharMap.hh"
#include "MathMLCharNode.hh"

class MathMLStretchyCharNode: public MathMLCharNode {
public:
  MathMLStretchyCharNode(Char ch);
  virtual void Fontify(const FontAttributes&);
  virtual void Render(const DrawingArea&);
  virtual ~MathMLStretchyCharNode();

  void SetDefaultLargeGlyph(bool);
  void DoMinimumLayout(void);
  void DoVerticalStretchyLayout(scaled, float, scaled, bool);
  void DoHorizontalStretchyLayout(scaled, bool);

  StretchId GetStretch(void) const { return stretch; }

  virtual bool IsStretchyChar(void) const;

private:
  void DoVerticalStretchyLayoutAux(scaled, bool);
  void DoHorizontalStretchyLayoutAux(scaled, bool);
  void RenderVerticalStretchyChar(const DrawingArea&, const GraphicsContext*, scaled, scaled);
  void RenderHorizontalStretchyChar(const DrawingArea&, const GraphicsContext*, scaled, scaled);

  StretchId   stretch;
  BoundingBox minimumBox;

  struct StretchyCharLayout {
    int simple;
    unsigned n;
    BoundingBox box[SC_REPEAT + 1];
    BoundingBox charBox;
  };

  StretchyCharLayout layout;
};

#define TO_STRETCHY_CHAR(node) (dynamic_cast<MathMLStretchyCharNode*>(node))

#endif
