// Copyright (C) 2000, Luca Padovani <luca.padovani@cs.unibo.it>.
// 
// This file is part of GtkMathView, a Gtk widget for MathML.
// 
// GtkMathView is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// GtkMathView is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with GtkMathView; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 
// For details, see the GtkMathView World-Wide-Web page,
// http://cs.unibo.it/~lpadovan/mml-widget, or send a mail to
// <luca.padovani@cs.unibo.it>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <assert.h>
#include <stdio.h>

#include "MathMLStringNode.hh"
#include "MathMLTokenElement.hh"

MathMLStringNode::MathMLStringNode(const String* c)
{
  assert(c != NULL);
  content = c;

  BreakId bid;
  int s;

  if (isNonMarkingChar(*c, &s, &bid) > 0) {
    isNonMarking = 1;
    SetSpacing(s);
    SetBreakability(bid);
  }
}

MathMLStringNode::~MathMLStringNode()
{
  delete content;
}

void
MathMLStringNode::Dump() const
{  
  PrintIndentation();
  printf("%s\n", content->ToStaticC());
}

bool
MathMLStringNode::IsString() const
{
  return true;
}

void
MathMLStringNode::Fontify(const FontAttributes& fontAttributes)
{
  // indeed, there is nothing to do, since this node is used
  // for plain strings only that don't need to be mapped
}

void
MathMLStringNode::DoLayout()
{
  assert(GetParent() != NULL);
  assert(GetParent()->IsToken());

  if (isNonMarking != 0) return;

  MathMLTokenElement* parent = TO_TOKEN(GetParent());
  assert(parent != NULL);

  assert(content != NULL);
  assert(parent->GetFont() != NULL);

  parent->GetFont()->StringBox(content->ToStaticC(), content->GetLength(), box);
}

void
MathMLStringNode::Render(const DrawingArea& area)
{
  assert(GetParent() != NULL);
  assert(GetParent()->IsToken());

  if (!HasDirtyChildren()) return;
  if (isNonMarking != 0) return;

  MathMLTokenElement* parent = TO_TOKEN(GetParent());
  assert(parent != NULL);

  assert(content != NULL);
  assert(parent->GetFont() != NULL);
  area.DrawString(GetParent()->GetGraphicsContext(), parent->GetFont(), GetX(), GetY(), content);

  ResetDirty();
}

bool
MathMLStringNode::HasDecimalPoint() const
{
  assert(content != NULL);

  if (isNonMarking != 0) return false;

  for (unsigned i = 0; i < content->GetLength(); i++)
    if (content->GetChar(i) == '.') return true;

  return false;
}

scaled
MathMLStringNode::GetDecimalPointEdge() const
{
  assert(content != NULL);
  assert(GetParent() != NULL);
  assert(GetParent()->IsToken());

  if (isNonMarking != 0) return GetX();

  MathMLTokenElement* parent = TO_TOKEN(GetParent());
  assert(parent != NULL);

  // let's find the position of the decimal point
  unsigned i;
  for (i = 0; i < content->GetLength() && content->GetChar(i) != '.'; i++) ;

  assert(parent->GetFont() != NULL);

  BoundingBox pBox;
  parent->GetFont()->StringBox(content->ToStaticC(), i, pBox);

  return GetX() + pBox.width;
}
